/*
 * Vorlesung ``Data Processing on Modern Hardware''
 *
 * Uebungsblatt 3
 *
 * Dieser Code basiert auf Code, der von Rene Mueller an
 * der ETH Zuerich entwickelt wurde.
 */

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <sys/time.h>
#include <time.h>

#include "schema.h"
#include "load.h"
#include "engine.h"

#define NUM_RUNS  100

int 
main(int argc, char **argv)
{
    uint32_t    numrows; 
    struct      timeval start, end;

    /* columns */
    uint32_t    *orderkey, *partkey, *suppkey;
    int32_t     *linenumber;
    int64_t     *quantity, *extendedprice, *discount, *tax;
    char        *returnflag, *linestatus;
    uint32_t    *shipdate, *commitdate, *receiptdate;
    char        *shipinstruct, *shipmode, *comment;

    /* operators */
    aggop_t     op_agg;
    scanop_t    scan_orderkey;
    int32_t     agg_result;


    /* load input data */
    printf("loading data file \"%s\"...\n", DATA_FILE); 
    gettimeofday(&start, NULL); 
    if (load_lineitems_columns(
                DATA_FILE, &numrows,
                &orderkey, &partkey, &suppkey, 
                &linenumber, &quantity, &extendedprice,
                &discount, &tax, &returnflag, 
                &linestatus, &shipdate, &commitdate, 
                &receiptdate, &shipinstruct, &shipmode, &comment))
    {
        fprintf(stderr, "error while reading data file: \"%s\"\n", DATA_FILE);
        exit(1); 
    }

    gettimeofday(&end, NULL); 

    printf("read %u rows in %d seconds\n", numrows, 
            (int)(end.tv_sec-start.tv_sec));

    /* setup query plan to the following query 
     *    SELECT MAX(orderkey)
     *    FROM lineitems
     */
    init_aggop(&op_agg, MAX, (anyop_t*)&scan_orderkey);
    init_scanop(&scan_orderkey, orderkey, numrows, 
            sizeof(uint32_t), 0, sizeof(uint32_t)); 

    printf("executing query...\n");

    /* fetch aggregate tuple from aggregation operator */
    if (op_agg.next(&agg_result, &op_agg) == -1)
        return -1;

    printf("result: %d\n", agg_result);


    /* cleanup before exit */
    free_aggop(&op_agg);          /* free operators */
    free_scanop(&scan_orderkey);
    free(orderkey);               /* free columns */
    free(partkey);
    free(suppkey);
    free(linenumber);
    free(quantity);
    free(extendedprice);
    free(discount);
    free(tax);
    free(returnflag); 
    free(linestatus);
    free(shipdate);
    free(commitdate);
    free(receiptdate);
    free(shipinstruct);
    free(shipmode);
    free(comment);
    return 0;
}

